/**
 * data-structure-typed
 *
 * @author Pablo Zeng
 * @copyright Copyright (c) 2022 Pablo Zeng <zrwusa@gmail.com>
 * @license MIT License
 */
import type { BTNRep, OptNodeOrNull, TreeMultiMapOptions } from '../../types';
import { RedBlackTree, RedBlackTreeNode } from './red-black-tree';
import { IBinaryTree } from '../../interfaces';
export declare class TreeMultiMapNode<K = any, V = any> extends RedBlackTreeNode<K, V[]> {
    /**
     * This TypeScript constructor initializes an object with a key of type K and an array of values of
     * type V.
     * @param {K} key - The `key` parameter is typically used to store a unique identifier or key for the
     * data being stored in the data structure. It helps in quickly accessing or retrieving the
     * associated value in the data structure.
     * @param {V[]} value - The `value` parameter in the constructor represents an array of values of
     * type `V`.
     */
    constructor(key: K, value: V[]);
    parent?: TreeMultiMapNode<K, V>;
    _left?: OptNodeOrNull<TreeMultiMapNode<K, V>>;
    get left(): OptNodeOrNull<TreeMultiMapNode<K, V>>;
    set left(v: OptNodeOrNull<TreeMultiMapNode<K, V>>);
    _right?: OptNodeOrNull<TreeMultiMapNode<K, V>>;
    get right(): OptNodeOrNull<TreeMultiMapNode<K, V>>;
    set right(v: OptNodeOrNull<TreeMultiMapNode<K, V>>);
}
/**
 *
 * @example
 * // Find elements in a range
 *     const tmm = new TreeMultiMap<number>([10, 5, 15, 3, 7, 12, 18]);
 *     console.log(tmm.search(new Range(5, 10))); // [5, 10, 7]
 *     console.log(tmm.search(new Range(4, 12))); // [5, 10, 12, 7]
 *     console.log(tmm.search(new Range(15, 20))); // [15, 18]
 */
export declare class TreeMultiMap<K = any, V = any, R = object, MK = any, MV = any, MR = object> extends RedBlackTree<K, V[], R, MK, MV[], MR> implements IBinaryTree<K, V[], R, MK, MV, MR> {
    /**
     * The constructor initializes an TreeMultiMap with the provided keys, nodes, entries, or raw data
     * and options.
     * @param keysNodesEntriesOrRaws - The `keysNodesEntriesOrRaws` parameter in the constructor is an
     * iterable that can contain either key-value pairs represented as `BTNRep<K, V[],
     * TreeMultiMapNode<K, V>>` or raw data represented as `R`. This parameter is used to initialize
     * the RedBlackTreeMulti
     * @param [options] - The `options` parameter in the constructor is of type
     * `TreeMultiMapOptions<K, V[], R>`. It is an optional parameter that allows you to specify
     * additional options for configuring the TreeMultiMap instance.
     */
    constructor(keysNodesEntriesOrRaws?: Iterable<BTNRep<K, V[], TreeMultiMapNode<K, V>> | R>, options?: TreeMultiMapOptions<K, V[], R>);
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     *
     * The `createTree` function in TypeScript overrides the default implementation to create a new
     * TreeMultiMap with specified options.
     * @param [options] - The `options` parameter in the `createTree` method is of type
     * `TreeMultiMapOptions<K, V[], R>`. This parameter allows you to pass additional configuration
     * options when creating a new `TreeMultiMap` instance. It includes properties such as
     * `iterationType`, `specifyComparable
     * @returns A new instance of `TreeMultiMap` is being returned, with an empty array as the initial
     * data and the provided options merged with the existing properties of the current object.
     */
    createTree(options?: TreeMultiMapOptions<K, V[], R>): TreeMultiMap<K, V, R, MK, MV, MR>;
    /**
     * Time Complexity: O(1)
     * Space Complexity: O(1)
     *
     * The function `createNode` overrides the method to create a new `TreeMultiMapNode` with a specified
     * key and an empty array of values.
     * @param {K} key - The `key` parameter in the `createNode` method represents the key of the node
     * that will be created in the TreeMultiMap data structure.
     * @returns A new instance of `TreeMultiMapNode<K, V>` is being returned, with the specified key and
     * an empty array as its value.
     */
    createNode(key: K): TreeMultiMapNode<K, V>;
    add(node: BTNRep<K, V[], TreeMultiMapNode<K, V>>): boolean;
    add(key: K, value: V): boolean;
    /**
     * Time Complexity: O(log n)
     * Space Complexity: O(log n)
     *
     * The function `deleteValue` removes a specific value from a key in a TreeMultiMap data structure
     * and deletes the entire node if no values are left for that key.
     * @param {BTNRep<K, V[], TreeMultiMapNode<K, V>> | K} keyNodeOrEntry - The `keyNodeOrEntry`
     * parameter in the `deleteValue` function can be either a `BTNRep` object containing a key and an
     * array of values, or just a key itself.
     * @param {V} value - The `value` parameter in the `deleteValue` function represents the specific
     * value that you want to remove from the multi-map data structure associated with a particular key.
     * The function checks if the value exists in the array of values associated with the key, and if
     * found, removes it from the array.
     * @returns The `deleteValue` function returns a boolean value - `true` if the specified `value` was
     * successfully deleted from the values associated with the `keyNodeOrEntry`, and `false` otherwise.
     */
    deleteValue(keyNodeOrEntry: BTNRep<K, V[], TreeMultiMapNode<K, V>> | K, value: V): boolean;
    /**
     * Time Complexity: O(n)
     * Space Complexity: O(n)
     *
     * The function `clone` overrides the default cloning behavior to create a deep copy of a tree
     * structure.
     * @returns The `cloned` object is being returned.
     */
    clone(): TreeMultiMap<K, V, R, MK, MV, MR>;
}
